<?php

namespace Drupal\whitelabel\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\user\UserInterface;
use Drupal\whitelabel\Entity\WhiteLabel;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Controller to help fetch the right white label form for a given user entity.
 */
class WhiteLabelGetEntityForm extends ControllerBase {

  /**
   * Holds the white label.
   *
   * @var \Drupal\whitelabel\WhiteLabelInterface
   */
  protected $whiteLabel;

  /**
   * WhiteLabelGetEntityForm constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManager $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManager $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * Helper function for obtaining the white label of a user.
   *
   * @param \Drupal\user\UserInterface $user
   *   The account to fetch the white label for.
   *
   * @return \Drupal\whitelabel\WhiteLabelInterface
   *   The white label object if found, an empty white label otherwise.
   */
  private function getWhiteLabel(UserInterface $user) {
    if (!isset($this->whiteLabel)) {
      $query = $this->entityTypeManager->getStorage('whitelabel')->getQuery();
      $query->condition('uid', $user->id());
      $entity_ids = $query->execute();

      if (!empty($entity_ids)) {
        $this->whiteLabel = WhiteLabel::load(reset($entity_ids));
      }
      else {
        $this->whiteLabel = WhiteLabel::create(['uid' => $user->id()]);
      }
    }

    return $this->whiteLabel;
  }

  /**
   * Retrieves the white label form that matches the given user.
   *
   * @param \Drupal\user\UserInterface $user
   *   The account to fetch the form for.
   *
   * @return array
   *   The render array of the form.
   */
  public function retrieveEntityForm(UserInterface $user) {
    $whitelabel = $this->getWhiteLabel($user);
    return $this->entityFormBuilder()->getForm($whitelabel);
  }

  /**
   * Checks if the user is allowed access to the edit form.
   *
   * @param \Drupal\user\UserInterface $user
   *   The account to check.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function checkFormAccess(UserInterface $user) {
    $whitelabel = $this->getWhiteLabel($user);
    $current_user = $this->currentUser();
    return $whitelabel->access('update', $current_user, TRUE);
  }

}
