<?php

namespace Drupal\whitelabel\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\file\FileInterface;
use Drupal\user\UserInterface;
use Drupal\whitelabel\WhiteLabelInterface;

/**
 * Defines the white label entity class.
 *
 * @ContentEntityType(
 *   id = "whitelabel",
 *   label = @Translation("White label"),
 *   label_plural = @Translation("white labels"),
 *   handlers = {
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "access" = "Drupal\whitelabel\WhiteLabelAccessControlHandler",
 *     "form" = {
 *       "default" = "Drupal\whitelabel\Form\WhiteLabelForm",
 *     },
 *    "route_provider" = {
 *       "html" = "Drupal\Core\Entity\Routing\DefaultHtmlRouteProvider",
 *     },
 *     "translation" = "Drupal\Core\Entity\EntityTranslationHandler"
 *   },
 *   admin_permission = "administer white label settings",
 *   base_table = "whitelabel",
 *   data_table = "whitelabel_field_data",
 *   revision_table = "whitelabel_revision",
 *   revision_data_table = "whitelabel_field_revision",
 *   translatable = TRUE,
 *   entity_keys = {
 *     "id" = "wid",
 *     "revision" = "vid",
 *     "label" = "token",
 *     "langcode" = "langcode",
 *     "uuid" = "uuid",
 *     "uid" = "uid",
 *   },
 *   field_ui_base_route = "entity.whitelabel.admin_form",
 *   links = {
 *     "edit-form" = "/user/{user}/whitelabel",
 *   }
 * )
 */
class WhiteLabel extends ContentEntityBase implements WhiteLabelInterface {

  /**
   * {@inheritdoc}
   */
  protected $cacheContexts = ['whitelabel'];

  /**
   * {@inheritdoc}
   *
   * Next to the default entity-type:id cache tag, also invalidate these.
   *
   * whitelabel_view is used to invalidate all white labels on a configuration
   * update.
   *
   * @see: Drupal\whitelabel\EventSubscriber\WhiteLabelCacheConfigInvalidator
   */
  protected $cacheTags = ['whitelabel_view'];

  /**
   * {@inheritdoc}
   */
  public function getOwner() {
    return $this->get('uid')->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwner(UserInterface $account) {
    $this->set('uid', $account->id());
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwnerId() {
    return $this->getEntityKey('uid');
  }

  /**
   * {@inheritdoc}
   */
  public function setOwnerId($uid) {
    $this->set('uid', $uid);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getToken() {
    return $this->get('token')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setToken($token) {
    $this->set('token', $token);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getName() {
    return $this->get('name')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setName($name) {
    $this->set('name', $name);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getSlogan() {
    return $this->get('slogan')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setSlogan($slogan) {
    $this->set('slogan', $slogan);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getLogo() {
    return $this->get('logo')->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function setLogo(FileInterface $file) {
    $this->set('logo', $file->id());
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getTheme() {
    return $this->get('theme')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setTheme($theme) {
    $this->set('theme', $theme);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getPalette() {
    $value = $this->get('palette')->getValue();
    return isset($value[0]['palette']) ? reset($value)['palette'] : [];
  }

  /**
   * {@inheritdoc}
   */
  public function setPalette(array $palette) {
    $this->set('palette', $palette);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getStylesheets() {
    $value = $this->get('stylesheets')->getValue();
    return isset($value[0]['stylesheets']) ? reset($value)['stylesheets'] : [];
  }

  /**
   * {@inheritdoc}
   */
  public function setStylesheets(array $stylesheets) {
    $this->set('stylesheets', $stylesheets);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['uid'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Owner'))
      ->setDescription(t('The username of the content author.'))
      ->setReadOnly(TRUE)
      ->setConstraints([
        'UniqueField' => [],
      ])
      ->setSetting('target_type', 'user')
      ->setDefaultValueCallback('Drupal\whitelabel\Entity\WhiteLabel::getCurrentUserId')
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'author',
        'weight' => 0,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['token'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Token'))
      ->setDescription(t('The unique token that will be appended to your white label URLs. Do not use you username! WARNING: Changing this later will break all exiting urls.'))
      ->setRequired(TRUE)
      ->setRevisionable(TRUE)
      ->setConstraints([
        'WhiteLabelTokenUnique' => [],
        'WhiteLabelNotUsername' => [],
      ])
      ->setSettings([
        'default_value' => '',
        'max_length' => 255,
        'text_processing' => 0,
      ])
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
        'weight' => -100,
      ])
      ->setDisplayOptions('form', [
        'type' => 'string',
        'weight' => -100,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['name'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Site name'))
      ->setDescription(t('The name of the site.'))
      ->setRevisionable(TRUE)
      ->setTranslatable(TRUE)
      ->setSettings([
        'default_value' => '',
        'max_length' => 255,
        'text_processing' => 0,
      ])
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
        'weight' => -10,
      ])
      ->setDisplayOptions('form', [
        'type' => 'string',
        'weight' => -10,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['slogan'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Site slogan'))
      ->setDescription(t('The slogan of the site.'))
      ->setRevisionable(TRUE)
      ->setTranslatable(TRUE)
      ->setSettings([
        'default_value' => '',
        'max_length' => 255,
        'text_processing' => 0,
      ])
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'string',
        'weight' => 0,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['logo'] = BaseFieldDefinition::create('image')
      ->setLabel(t('Site logo'))
      ->setDescription(t('Custom site logo'))
      ->setRevisionable(TRUE)
      ->setSettings([
        'file_directory' => 'whitelabel',
        'alt_field' => FALSE,
        'alt_field_required' => FALSE,
        'file_extensions' => 'png gif jpg jpeg svg',
        'max_resolution' => '640x480',
      ])
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'default',
        'weight' => 10,
      ])
      ->setDisplayOptions('form', [
        'type' => 'image_image',
        'weight' => 10,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    // Cannot be named `theme` because color adds that to the form as well.
    $fields['theme'] = BaseFieldDefinition::create('list_string')
      ->setLabel(t('Site theme'))
      ->setDescription(t('Custom site theme'))
      ->setCardinality(1)
      ->setRevisionable(TRUE)
      ->setSettings([
        'allowed_values' => whitelabel_load_available_themes(),
        'default_value' => '',
      ])
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
        'weight' => 5,
      ])
      ->setDisplayOptions('form', [
        'type' => 'options_select',
        'weight' => 5,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['palette'] = BaseFieldDefinition::create('map')
      ->setLabel(t('Color palette'))
      ->setDescription(t('The color palette of this white label.'))
      ->setQueryable(FALSE)
      ->setRevisionable(TRUE);

    $fields['stylesheets'] = BaseFieldDefinition::create('map')
      ->setLabel(t('Stylesheets'))
      ->setDescription(t('The stylesheets this white label requires.'))
      ->setQueryable(FALSE)
      ->setRevisionable(TRUE);

    return $fields;
  }

  /**
   * Default value callback for 'uid' base field definition.
   *
   * @see ::baseFieldDefinitions()
   *
   * @return array
   *   An array of default values.
   */
  public static function getCurrentUserId() {
    return [\Drupal::currentUser()->id()];
  }

}
