<?php

namespace Drupal\whitelabel\Form;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Asset\CssOptimizer;
use Drupal\Core\Entity\ContentEntityForm;
use Drupal\Core\Entity\EntityManagerInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Theme\ThemeManagerInterface;
use Drupal\user\UserInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form controller for the white label entity edit forms.
 */
class WhiteLabelForm extends ContentEntityForm {

  /**
   * The theme manager.
   *
   * @var \Drupal\Core\Theme\ThemeManagerInterface
   */
  protected $themeManager;

  /**
   * The file system.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * Constructs a ContentEntityForm object.
   *
   * @param \Drupal\Core\Entity\EntityManagerInterface $entity_manager
   *   The entity manager.
   * @param \Drupal\Core\Theme\ThemeManagerInterface $theme_manager
   *   The theme mananger.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entity_type_bundle_info
   *   The entity type bundle service.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time service.
   */
  public function __construct(EntityManagerInterface $entity_manager,
                              ThemeManagerInterface $theme_manager,
                              FileSystemInterface $file_system,
                              EntityTypeBundleInfoInterface $entity_type_bundle_info = NULL,
                              TimeInterface $time = NULL) {
    parent::__construct($entity_manager, $entity_type_bundle_info, $time);
    $this->themeManager = $theme_manager;
    $this->fileSystem = $file_system;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity.manager'),
      $container->get('theme.manager'),
      $container->get('file_system'),
      $container->get('entity_type.bundle.info'),
      $container->get('datetime.time')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);

    /* @var \Drupal\whitelabel\Entity\WhiteLabel $whitelabel */
    $whitelabel = $this->entity;

    // Add ajax callback to the form.
    $form['theme']['widget']['#ajax'] = [
      'callback' => [$this, 'ajaxCallback'],
      'wrapper' => 'color_scheme_form',
    ];

    // TODO: Get name of the selected theme and keep in sync with AJAX.
    // Get name of the current theme.
    $theme = $form_state->getValue('theme') ?: $whitelabel->getTheme() ?: $this->themeManager->getActiveTheme()->getName();

    while (is_array($theme)) {
      $theme = reset($theme);
    }

    // Attach the color form.
    if ($this->moduleHandler->moduleExists('color') && color_get_info($theme) && function_exists('gd_info')) {
      $form['color'] = [
        '#type' => 'fieldset',
        '#title' => $this->t('Color scheme'),
        '#weight' => 15,
        '#attributes' => ['id' => 'color_scheme_form'],
        '#theme' => 'color_scheme_form',
        '#access' => $this->config('whitelabel.settings')->get('site_colors'),
      ];
      $form['color'] += whitelabel_color_scheme_form($form, $form_state, $theme, $whitelabel);

      // Attach whitelabel_color.js (customized color.js).
      $form['color']['scheme']['#attached']['library'][] = 'whitelabel/whitelabel.form';

      // Add validators (from color module).
      $form['#validate'][] = 'color_scheme_form_validate';
    }

    return $form;
  }

  /**
   * Callback to update parts of the form after an AJAX request.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   The changed portion of the form.
   */
  public function ajaxCallback(array $form, FormStateInterface $form_state) {
    return $form['color'];
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    /* @var \Drupal\whitelabel\Entity\WhiteLabel $whitelabel */
    $whitelabel = $this->getEntity();

    // Get palette as array.
    $palette = $form_state->getValue('palette');
    $whitelabel->setPalette(['palette' => $palette]);

    // Delete old files.
    $files = $whitelabel->getStylesheets();
    if (isset($files)) {
      foreach ($files as $file) {
        @$this->fileSystem->unlink($file);
      }
    }
    if (isset($file) && $file = dirname($file)) {
      @$this->fileSystem->unlink($file);
    }

    // Avoid color settings spilling over to theme settings.
    $color_settings = ['theme', 'palette', 'scheme'];
    if ($form_state->hasValue('info')) {
      $color_settings[] = 'info';
    }
    foreach ($color_settings as $setting_name) {
      ${$setting_name} = $form_state->getValue($setting_name);
      $form_state->unsetValue($setting_name);
    }
    if (!isset($info)) {
      return;
    }

    // Prepare target locations for generated files.
    $id = $whitelabel->getToken();
    $paths['color'] = 'public://whitelabel';
    $paths['target'] = $paths['color'] . '/' . $id;
    foreach ($paths as $path) {
      file_prepare_directory($path, FILE_CREATE_DIRECTORY);
    }
    $paths['target'] = $paths['target'] . '/';
    $paths['id'] = $id;
    // AUTO CREATED ON LINE 80.
    $paths['source'] = drupal_get_path('theme', $theme) . '/';
    $paths['files'] = $paths['map'] = [];

    // Rewrite theme stylesheets.
    $css = [];
    foreach ($info['css'] as $stylesheet) {
      // Build a temporary array with CSS files.
      $files = [];
      if (file_exists($paths['source'] . $stylesheet)) {
        $files[] = $stylesheet;
      }

      foreach ($files as $file) {
        $css_optimizer = new CssOptimizer();
        // Aggregate @imports recursively for each configured top level CSS file
        // without optimization. Aggregation and optimization will be
        // handled by drupal_build_css_cache() only.
        $style = $css_optimizer->loadFile($paths['source'] . $file, FALSE);

        // Return the path to where this CSS file originated from, stripping
        // off the name of the file at the end of the path.
        $css_optimizer->rewriteFileURIBasePath = base_path() . dirname($paths['source'] . $file) . '/';

        // Prefix all paths within this CSS file, ignoring absolute paths.
        $style = preg_replace_callback('/url\([\'"]?(?![a-z]+:|\/+)([^\'")]+)[\'"]?\)/i', [$css_optimizer, 'rewriteFileURI'], $style);

        // Rewrite stylesheet with new colors.
        $style = _color_rewrite_stylesheet($theme, $info, $paths, $palette, $style);

        $base_file = $this->fileSystem->basename($file);
        $css[] = $paths['target'] . $base_file;
        _color_save_stylesheet($paths['target'] . $base_file, $style, $paths);
      }
    }

    $whitelabel->setStylesheets(['stylesheets' => $css]);
    $whitelabel->save();

    drupal_set_message($this->t('The changes have been saved.'));
  }

  /**
   * Checks access for the white label form.
   *
   * @param \Drupal\user\UserInterface $user
   *   (optional) The owner of the white label.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function checkAccess(UserInterface $user = NULL) {
    return shortcut_set_switch_access($user);
  }

}
