<?php

namespace Drupal\Tests\whitelabel\Functional;

use Drupal\file\Entity\File;

/**
 * Tests if the form fields and rendered values show and hide as configured.
 *
 * @group whitelabel
 */
class WhiteLabelFeatureToggleTest extends WhiteLabelTestBase {

  /**
   * {@inheritdoc}
   */
  public static $modules = [
    'color',
    'block',
    'whitelabel_test',
  ];

  /**
   * Array of all config fields that toggle form/page elements.
   *
   * @var array
   */
  private $configDefaults = [
    'site_name' => FALSE,
    'site_name_display' => FALSE,
    'site_slogan' => FALSE,
    'site_logo' => FALSE,
    'site_theme' => FALSE,
    'site_colors' => FALSE,
    // 'site_admin_theme', This does not appear on the entity form.
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();

    // Make sure everything is disabled by default.
    $this->config('whitelabel.settings')
      ->setData($this->configDefaults)
      ->save();
  }

  /**
   * Test to see if the form displays the right fields.
   */
  public function testForm() {
    // We need a second theme to be installed to have it show up on the form.
    $this->container->get('theme_installer')->install(['bartik']);

    // Remove the logo so we can assert the display of the form field.
    // Enable a theme with color integration.
    $this->whiteLabel
      ->setLogo(File::create())
      ->setTheme('bartik')
      ->save();

    foreach (array_keys($this->configDefaults) as $config_key) {
      $this->config('whitelabel.settings')
        // Disable all features.
        ->setData($this->configDefaults)
        // Enable the feature we want to test.
        ->set($config_key, TRUE)
        ->save();

      $this->drupalGet('user/' . $this->whiteLabel->getOwnerId() . '/whitelabel');;

      // Token should always be visible.
      $this->assertSession()->fieldExists("token[0][value]");

      $config_key == 'site_name' ?
        $this->assertSession()->fieldExists("name[0][value]") :
        $this->assertSession()->fieldNotExists("name[0][value]");

      // TODO: site_name_display.

      $config_key == 'site_slogan' ?
        $this->assertSession()->fieldExists("slogan[0][value]") :
        $this->assertSession()->fieldNotExists("slogan[0][value]");

      $config_key == 'site_logo' ?
        $this->assertSession()->fieldExists("edit-logo-0-upload") :
        $this->assertSession()->fieldNotExists("edit-logo-0-upload");

      $config_key == 'site_theme' ?
        $this->assertSession()->fieldExists("theme") :
        $this->assertSession()->fieldNotExists("theme");

      $config_key == 'site_colors' ?
        $this->assertSession()->elementExists('css', '#color_scheme_form') :
        $this->assertSession()->elementNotExists('css', '#color_scheme_form');
    }
  }

  /**
   * Test to see if the page displays the right values.
   *
   * Application of themes and color schemes have their own test.
   *
   * @see \Drupal\Tests\whitelabel\Functional\WhiteLabelStyleSheetTest
   * @see \Drupal\Tests\whitelabel\Functional\WhiteLabelThemeNegotiatorTest
   */
  public function testPage() {
    // Place branding block with site name and slogan into header region.
    $this->drupalPlaceBlock('system_branding_block', [
      'region' => 'header',
    ]);

    // Set a white label.
    $this->setCurrentWhiteLabel($this->whiteLabel);

    foreach (array_keys($this->configDefaults) as $config_key) {
      $this->config('whitelabel.settings')
        // Disable all features.
        ->setData($this->configDefaults)
        // Enable the feature we want to test.
        ->set($config_key, TRUE)
        ->save();

      // Visit the front page.
      $this->drupalGet('<front>');

      $config_key == 'site_name' ?
        $this->inBrandingBlock($this->whiteLabel->getName()) :
        $this->notInBrandingBlock($this->whiteLabel->getName());

      // TODO: Site name display.

      $config_key == 'site_slogan' ?
        $this->inBrandingBlock($this->whiteLabel->getSlogan()) :
        $this->notInBrandingBlock($this->whiteLabel->getSlogan());

      $config_key == 'site_logo' ?
        $this->inImagePath(file_url_transform_relative(file_create_url($this->whiteLabel->getLogo()->getFileUri()))) :
        $this->notInImagePath(file_url_transform_relative(file_create_url($this->whiteLabel->getLogo()->getFileUri())));
    }
  }

}
